import PropTypes from "prop-types";
import { CSSProperties, SyntheticEvent } from "react";
import { ColumnDescription, TableColumnFilterProps } from "react-bootstrap-table-nextgen";
import DateFilter from "./src/components/date";
import MultiSelectFilter from "./src/components/multiselect";
import NumberFilter from "./src/components/number";
import SelectFilter from "./src/components/select";
import TextFilter from "./src/components/text";
export declare enum FILTER_TYPES {
    TEXT = "TEXT",
    SELECT = "SELECT",
    MULTISELECT = "MULTISELECT",
    NUMBER = "NUMBER",
    DATE = "DATE"
}
export declare const FILTER_DELAY = 500;
/**
 * Filter comparators used for table filters
 */
export declare const LIKE = "LIKE";
export declare const EQ = "=";
export declare const NE = "!=";
export declare const GT = ">";
export declare const GE = ">=";
export declare const LT = "<";
export declare const LE = "<=";
export type Comparator = string;
export declare const ComparatorNumber = 7;
export type TextFilterProps<T extends object = any> = TableColumnFilterProps<string, T> & Partial<{
    caseSensitive: boolean;
    comparator: Comparator;
    onClick: ((e: SyntheticEvent) => void) | undefined;
    filterState?: any;
    column: any;
}>;
export declare const textFilter: (props?: Partial<TextFilterProps>) => {
    Filter: typeof TextFilter;
    props: Partial<TextFilterProps<any>>;
};
export type SelectFilterOptions = {
    [index: string]: string;
} | Array<{
    value: number;
    label: string;
}>;
export type SelectFilterProps<T extends object = any> = TableColumnFilterProps<string, T> & {
    options: SelectFilterOptions | ((column: ColumnDescription<T>) => SelectFilterOptions);
    comparator?: Comparator | undefined;
    withoutEmptyOption?: boolean | undefined;
    filterState?: any;
    column: any;
    caseSensitive?: boolean;
};
export declare const selectFilter: (props?: Partial<SelectFilterProps>) => {
    Filter: typeof SelectFilter;
    props: Partial<SelectFilterProps<any>>;
};
export type MultiSelectFilterOptions = {
    [index: string]: string;
};
export type MultiSelectFilterProps<T extends object = any> = TableColumnFilterProps<string[], T> & {
    options: MultiSelectFilterOptions | (() => MultiSelectFilterOptions);
    comparator?: Comparator | undefined;
    withoutEmptyOption?: boolean | undefined;
    filterState?: any;
    column: any;
    caseSensitive?: boolean;
};
export declare const multiSelectFilter: (props?: Partial<MultiSelectFilterProps>) => {
    Filter: typeof MultiSelectFilter;
    props: Partial<MultiSelectFilterProps<any>>;
};
export type NumberFilterProps<T extends object = any> = TableColumnFilterProps<{
    number: number | "";
    comparator: Comparator;
}, T> & {
    options?: number[] | undefined;
    comparators?: Comparator[] | undefined;
    withoutEmptyComparatorOption?: boolean | undefined;
    withoutEmptyNumberOption?: boolean | undefined;
    comparatorClassName?: string | undefined;
    numberClassName?: string | undefined;
    comparatorStyle?: CSSProperties | undefined;
    numberStyle?: CSSProperties | undefined;
    defaultValue?: {
        number: number;
        comparator: Comparator;
    } | undefined;
    filterState?: any;
    column: any;
};
export declare const numberFilter: (props?: Partial<NumberFilterProps>) => {
    Filter: typeof NumberFilter;
    props: Partial<NumberFilterProps<any>>;
};
export type DateFilterProps<T extends object = any> = TableColumnFilterProps<Date, T> & {
    withoutEmptyComparatorOption?: boolean | undefined;
    defaultValue?: {
        date?: Date;
        comparator?: Comparator;
    } | undefined;
    comparators?: Comparator[] | undefined;
    comparatorClassName?: string | undefined;
    dateClassName?: string | undefined;
    comparatorStyle?: CSSProperties | undefined;
    dateStyle?: CSSProperties | undefined;
    filterState?: any;
    column: any;
};
export declare const dateFilter: (props?: Partial<DateFilterProps>) => {
    Filter: typeof DateFilter;
    props: Partial<DateFilterProps<any>>;
};
export type CustomFilterProps = {
    type?: string | FILTER_TYPES | undefined;
    comparator?: Comparator | undefined;
    caseSensitive?: boolean | undefined;
};
export declare const customFilter: (props?: Partial<CustomFilterProps>) => {
    Filter: () => void;
    props: Partial<CustomFilterProps>;
};
/**
 * declaration for table filter sub module
 */
export type FilterFactoryProps<T extends object = any> = {
    afterFilter?: ((newResult: T[], newFilters?: unknown[]) => void) | undefined;
};
declare const _default: (options?: {}) => {
    createContext: (_: any, isRemoteFiltering: () => boolean, handleFilterChange: (filters: any) => void) => {
        Provider: {
            new (props: import("./src/context").FilterProviderProps): {
                currFilters: {
                    [key: string]: any;
                };
                clearFilters: {
                    [key: string]: any;
                };
                data: any[];
                isEmitDataChange: boolean;
                componentDidMount(): void;
                onFilter(column: any, filterType: any, initialize?: boolean): (filterVal: any) => void;
                onExternalFilter(column: any, filterType: any): (value: any) => void;
                getFiltered(): any[];
                doFilter(props: import("./src/context").FilterProviderProps, ignoreEmitDataChange?: boolean): void;
                render(): import("react").JSX.Element;
                context: unknown;
                setState<K extends never>(state: {} | ((prevState: Readonly<{}>, props: Readonly<import("./src/context").FilterProviderProps>) => {} | Pick<{}, K> | null) | Pick<{}, K> | null, callback?: (() => void) | undefined): void;
                forceUpdate(callback?: (() => void) | undefined): void;
                readonly props: Readonly<import("./src/context").FilterProviderProps>;
                state: Readonly<{}>;
                refs: {
                    [key: string]: import("react").ReactInstance;
                };
                shouldComponentUpdate?(nextProps: Readonly<import("./src/context").FilterProviderProps>, nextState: Readonly<{}>, nextContext: any): boolean;
                componentWillUnmount?(): void;
                componentDidCatch?(error: Error, errorInfo: import("react").ErrorInfo): void;
                getSnapshotBeforeUpdate?(prevProps: Readonly<import("./src/context").FilterProviderProps>, prevState: Readonly<{}>): any;
                componentDidUpdate?(prevProps: Readonly<import("./src/context").FilterProviderProps>, prevState: Readonly<{}>, snapshot?: any): void;
                componentWillMount?(): void;
                UNSAFE_componentWillMount?(): void;
                componentWillReceiveProps?(nextProps: Readonly<import("./src/context").FilterProviderProps>, nextContext: any): void;
                UNSAFE_componentWillReceiveProps?(nextProps: Readonly<import("./src/context").FilterProviderProps>, nextContext: any): void;
                componentWillUpdate?(nextProps: Readonly<import("./src/context").FilterProviderProps>, nextState: Readonly<{}>, nextContext: any): void;
                UNSAFE_componentWillUpdate?(nextProps: Readonly<import("./src/context").FilterProviderProps>, nextState: Readonly<{}>, nextContext: any): void;
            };
            propTypes: {
                data: PropTypes.Validator<any[]>;
                columns: PropTypes.Validator<any[]>;
                dataChangeListener: PropTypes.Requireable<object>;
            };
            contextType?: import("react").Context<any> | undefined;
        };
        Consumer: import("react").Consumer<any>;
    };
    options: {};
};
export default _default;
