"use strict";

function _typeof(o) { "@babel/helpers - typeof"; return _typeof = "function" == typeof Symbol && "symbol" == typeof Symbol.iterator ? function (o) { return typeof o; } : function (o) { return o && "function" == typeof Symbol && o.constructor === Symbol && o !== Symbol.prototype ? "symbol" : typeof o; }, _typeof(o); }
Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.filters = exports.filterFactory = exports.filterByArray = exports.filterByDate = exports.filterByNumber = exports.filterByText = void 0;
/* eslint eqeqeq: 0 */
/* eslint no-console: 0 */
var __1 = require("..");
var filterByText = function filterByText(_) {
  return function (data, dataField, _ref, customFilterValue) {
    var _ref$filterVal = _ref.filterVal,
      userInput = _ref$filterVal === void 0 ? "" : _ref$filterVal,
      _ref$comparator = _ref.comparator,
      comparator = _ref$comparator === void 0 ? __1.LIKE : _ref$comparator,
      caseSensitive = _ref.caseSensitive;
    // make sure filter value to be a string
    var filterVal = userInput.toString();
    return data.filter(function (row) {
      var cell = _.get(row, dataField);
      if (customFilterValue) {
        cell = customFilterValue(cell, row);
      }
      var cellStr = _.isDefined(cell) ? cell.toString() : "";
      if (comparator === __1.EQ) {
        return cellStr === filterVal;
      }
      if (caseSensitive) {
        return cellStr.includes(filterVal);
      }
      return cellStr.toLocaleUpperCase().indexOf(filterVal.toLocaleUpperCase()) !== -1;
    });
  };
};
exports.filterByText = filterByText;
var filterByNumber = function filterByNumber(_) {
  return function (data, dataField, _ref2, customFilterValue) {
    var _ref2$filterVal = _ref2.filterVal,
      comparator = _ref2$filterVal.comparator,
      number = _ref2$filterVal.number;
    return data.filter(function (row) {
      if (number === "" || !comparator) return true;
      var cell = _.get(row, dataField);
      if (customFilterValue) {
        cell = customFilterValue(cell, row);
      }
      switch (comparator) {
        case __1.EQ:
          {
            return cell == number;
          }
        case __1.GT:
          {
            return cell > number;
          }
        case __1.GE:
          {
            return cell >= number;
          }
        case __1.LT:
          {
            return cell < number;
          }
        case __1.LE:
          {
            return cell <= number;
          }
        case __1.NE:
          {
            return cell != number;
          }
        default:
          {
            console.error("Number comparator provided is not supported");
            return true;
          }
      }
    });
  };
};
exports.filterByNumber = filterByNumber;
var filterByDate = function filterByDate(_) {
  return function (data, dataField, _ref3, customFilterValue) {
    var _ref3$filterVal = _ref3.filterVal,
      comparator = _ref3$filterVal.comparator,
      date = _ref3$filterVal.date;
    if (!date || !comparator) return data;
    var filterDate = date.getUTCDate();
    var filterMonth = date.getUTCMonth();
    var filterYear = date.getUTCFullYear();
    return data.filter(function (row) {
      var valid = true;
      var cell = _.get(row, dataField);
      if (customFilterValue) {
        cell = customFilterValue(cell, row);
      }
      if (_typeof(cell) !== "object") {
        cell = new Date(cell);
      }
      var targetDate = cell.getUTCDate();
      var targetMonth = cell.getUTCMonth();
      var targetYear = cell.getUTCFullYear();
      switch (comparator) {
        case __1.EQ:
          {
            if (filterDate !== targetDate || filterMonth !== targetMonth || filterYear !== targetYear) {
              valid = false;
            }
            break;
          }
        case __1.GT:
          {
            if (cell <= date) {
              valid = false;
            }
            break;
          }
        case __1.GE:
          {
            if (targetYear < filterYear) {
              valid = false;
            } else if (targetYear === filterYear && targetMonth < filterMonth) {
              valid = false;
            } else if (targetYear === filterYear && targetMonth === filterMonth && targetDate < filterDate) {
              valid = false;
            }
            break;
          }
        case __1.LT:
          {
            if (cell >= date) {
              valid = false;
            }
            break;
          }
        case __1.LE:
          {
            if (targetYear > filterYear) {
              valid = false;
            } else if (targetYear === filterYear && targetMonth > filterMonth) {
              valid = false;
            } else if (targetYear === filterYear && targetMonth === filterMonth && targetDate > filterDate) {
              valid = false;
            }
            break;
          }
        case __1.NE:
          {
            if (filterDate === targetDate && filterMonth === targetMonth && filterYear === targetYear) {
              valid = false;
            }
            break;
          }
        default:
          {
            console.error("Date comparator provided is not supported");
            break;
          }
      }
      return valid;
    });
  };
};
exports.filterByDate = filterByDate;
var filterByArray = function filterByArray(_) {
  return function (data, dataField, _ref4) {
    var filterVal = _ref4.filterVal,
      comparator = _ref4.comparator;
    if (filterVal.length === 0) return data;
    var refinedFilterVal = filterVal.filter(function (x) {
      return _.isDefined(x);
    }).map(function (x) {
      return x.toString();
    });
    return data.filter(function (row) {
      var cell = _.get(row, dataField);
      var cellStr = _.isDefined(cell) ? cell.toString() : "";
      if (comparator === __1.EQ) {
        return refinedFilterVal.indexOf(cellStr) !== -1;
      }
      cellStr = cellStr.toLocaleUpperCase();
      return refinedFilterVal.some(function (item) {
        return cellStr.indexOf(item.toLocaleUpperCase()) !== -1;
      });
    });
  };
};
exports.filterByArray = filterByArray;
var filterFactory = function filterFactory(_) {
  return function (filterType) {
    switch (filterType) {
      case __1.FILTER_TYPES.MULTISELECT:
        return (0, exports.filterByArray)(_);
      case __1.FILTER_TYPES.NUMBER:
        return (0, exports.filterByNumber)(_);
      case __1.FILTER_TYPES.DATE:
        return (0, exports.filterByDate)(_);
      case __1.FILTER_TYPES.TEXT:
      case __1.FILTER_TYPES.SELECT:
      default:
        // Use `text` filter as default filter
        return (0, exports.filterByText)(_);
    }
  };
};
exports.filterFactory = filterFactory;
var filters = function filters(data, columns, _) {
  return function (currFilters) {
    var clearFilters = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
    var factory = (0, exports.filterFactory)(_);
    var filterState = Object.assign(Object.assign({}, clearFilters), currFilters);
    var result = data;
    var filterFn;
    Object.keys(filterState).forEach(function (dataField) {
      var currentResult;
      var filterValue;
      var customFilter;
      columns = columns !== null && columns !== void 0 ? columns : [];
      for (var i = 0; i < columns.length; i += 1) {
        if (columns[i].dataField === dataField) {
          filterValue = columns[i].filterValue;
          if (columns[i].filter) {
            customFilter = columns[i].filter.props.onFilter;
          }
          break;
        }
      }
      if (clearFilters[dataField] && customFilter) {
        currentResult = customFilter(clearFilters[dataField].filterVal, result);
        if (typeof currentResult !== "undefined") {
          result = currentResult;
        }
      } else {
        var filterObj = filterState[dataField];
        filterFn = factory(filterObj.filterType);
        if (customFilter) {
          currentResult = customFilter(filterObj.filterVal, result);
        }
        if (typeof currentResult === "undefined") {
          result = filterFn(result, dataField, filterObj, filterValue);
        } else {
          result = currentResult;
        }
      }
    });
    return result;
  };
};
exports.filters = filters;